/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: EjbJarDeployableMetadataFactory.java 2950 2008-04-25 09:14:23Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ejbjar.impl;

import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.ee.deploy.api.deployable.metadata.IDeployableMetadataFactory;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.EjbJarDeployableMetadataConfigurator;
import org.ow2.util.scan.api.ArchiveScanner;
import org.ow2.util.scan.api.ArchiveScannerFactory;
import org.ow2.util.scan.api.ScanException;

/**
 * Factory to contruct ejb jar deployable metadata.
 * @param <E> ejb-jar type (EJB3, EJB2.1)
 * @author Gael Lalire
 */
public class EjbJarDeployableMetadataFactory<E extends EJBDeployable<E>> implements
        IDeployableMetadataFactory<E, IEjbJarDeployableMetadata<E>> {

    /**
     * Archive scanner factory.
     */
    private ArchiveScannerFactory archiveScannerFactory;

    /**
     * Constructor.
     * @throws ScanException if no archive scanner factory can be found
     */
    public EjbJarDeployableMetadataFactory() throws ScanException {
        this(ArchiveScannerFactory.getInstance());
    }

    /**
     * Constructor with an archive scanner factory.
     * @param archiveScannerFactory the archive scanner factory
     */
    public EjbJarDeployableMetadataFactory(final ArchiveScannerFactory archiveScannerFactory) {
        this.archiveScannerFactory = archiveScannerFactory;
    }

    /**
     * @param ejb3Deployable an ejb3 deployable
     * @return metadata constructed using ejb3Deployable
     * @throws DeployableMetadataException if metadata can not be created
     */
    public IEjbJarDeployableMetadata<E> createDeployableMetadata(final E ejb3Deployable)
            throws DeployableMetadataException {

        IArchive archive = ejb3Deployable.getArchive();

        // create an archive scanner (for annotation)
        ArchiveScanner archiveScanner = archiveScannerFactory.createArchiveScanner(archive);


        EjbJarDeployableMetadataConfigurator<E> ejbJarArchiveMetadataConfigurator = new EjbJarDeployableMetadataConfigurator<E>(
                ejb3Deployable);
        IEjbJarDeployableMetadata<E> ejbJarArchiveMetadata = ejbJarArchiveMetadataConfigurator.getEjbJarArchiveMetadata();
        archiveScanner.addArchiveConfigurator(ejbJarArchiveMetadataConfigurator);

        try {
            archiveScanner.scanAll();
        } catch (ScanException e) {
            throw new DeployableMetadataException("Unable to scan all archive", e);
        }

        return ejbJarArchiveMetadata;
    }

    /**
     * @return the class returned by creation of metadata : IEJB3DeployableMetadata.class
     */
    @SuppressWarnings("unchecked")
    public Class<IEjbJarDeployableMetadata<E>> getDeployableMetadataClass() {
        return (Class<IEjbJarDeployableMetadata<E>>) (Class<?>) IEjbJarDeployableMetadata.class;
    }

}
