package org.ow2.util.ee.metadata.ejbjar.impl.configurator;

import org.ow2.util.ee.deploy.api.deployable.EJBDeployable;
import org.ow2.util.ee.metadata.common.impl.configurator.CommonClassMetadataConfigurator;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityPermitAllVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxAnnotationSecurityRolesAllowedVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxEjbTransactionAttributeVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorExcludeDefaultInterceptorsVisitor;
import org.ow2.util.ee.metadata.common.impl.configurator.visitor.JavaxInterceptorInterceptorsVisitor;
import org.ow2.util.ee.metadata.ejbjar.api.IEjbJarDeployableMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarClassMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarFieldMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.EjbJarMethodMetadata;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxAnnotationSecurityDeclareRolesVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxAnnotationSecurityRunAs;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbApplicationExceptionVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbLocalHomeVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbLocalVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbMessageDrivenVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbRemoteHomeVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbRemoteVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbStatefulVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbStatelessVisitor;
import org.ow2.util.ee.metadata.ejbjar.impl.configurator.visitor.clazz.JavaxEjbTransactionManagementVisitor;
import org.ow2.util.scan.api.configurator.IFieldConfigurator;
import org.ow2.util.scan.api.configurator.IMethodConfigurator;
import org.ow2.util.scan.api.metadata.structures.JClass;
import org.ow2.util.scan.api.metadata.structures.JField;
import org.ow2.util.scan.api.metadata.structures.JMethod;


/**
 * EjbJarClassMetadataConfigurator.
 * @param <E> deployable type
 * @author Gael Lalire
 */
public class EjbJarClassMetadataConfigurator<E extends EJBDeployable<E>> extends
        CommonClassMetadataConfigurator<EjbJarClassMetadata<E>, EjbJarMethodMetadata<E>, EjbJarFieldMetadata<E>> {


    /**
     * annotationParsingDesactived.
     */
    private boolean annotationParsingDesactived;

    /**
     * EjbJarClassMetadataConfigurator.
     * @param jClass signature
     * @param ejbJarArchiveMetadata parent
     * @param annotationParsingDesactived annotationParsingDesactived
     */
    public EjbJarClassMetadataConfigurator(final JClass jClass, final IEjbJarDeployableMetadata<E> ejbJarArchiveMetadata,
            final boolean annotationParsingDesactived) {
        super(new EjbJarClassMetadata<E>(jClass, ejbJarArchiveMetadata));
        this.annotationParsingDesactived = annotationParsingDesactived;
        if (!annotationParsingDesactived) {
            registerAnnotationVisitor();
        }
    }

    /**
     * register annotation visitor.
     */
    @Override
    protected void registerAnnotationVisitor() {

        super.registerAnnotationVisitor();

        EjbJarClassMetadata<E> ejbJarClassMetadata = getClassMetadata();

        // add @Local
        getAnnotationVisitors().put(JavaxEjbLocalVisitor.TYPE, new JavaxEjbLocalVisitor<E>(ejbJarClassMetadata));

        // add @Remote
        getAnnotationVisitors().put(JavaxEjbRemoteVisitor.TYPE, new JavaxEjbRemoteVisitor<E>(ejbJarClassMetadata));

        // add @Stateless
        getAnnotationVisitors().put(JavaxEjbStatelessVisitor.TYPE,
                new JavaxEjbStatelessVisitor<E>(ejbJarClassMetadata));

        // add @Stateful
        getAnnotationVisitors().put(JavaxEjbStatefulVisitor.TYPE, new JavaxEjbStatefulVisitor<E>(ejbJarClassMetadata));

        // add @MessageDriven
        getAnnotationVisitors().put(JavaxEjbMessageDrivenVisitor.TYPE, new JavaxEjbMessageDrivenVisitor<E>(
                ejbJarClassMetadata));

        // add @LocalHome
        getAnnotationVisitors().put(JavaxEjbLocalHomeVisitor.TYPE,
                new JavaxEjbLocalHomeVisitor<E>(ejbJarClassMetadata));

        // add @RemoteHome
        getAnnotationVisitors().put(JavaxEjbRemoteHomeVisitor.TYPE, new JavaxEjbRemoteHomeVisitor<E>(
                ejbJarClassMetadata));

        // add @TransactionManagement
        getAnnotationVisitors().put(JavaxEjbTransactionManagementVisitor.TYPE,
                new JavaxEjbTransactionManagementVisitor<E>(ejbJarClassMetadata));

        // add @TransactionAttribute
        getAnnotationVisitors().put(JavaxEjbTransactionAttributeVisitor.TYPE,
                new JavaxEjbTransactionAttributeVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @Interceptors
        getAnnotationVisitors().put(JavaxInterceptorInterceptorsVisitor.TYPE,
                new JavaxInterceptorInterceptorsVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @ApplicationException
        getAnnotationVisitors().put(JavaxEjbApplicationExceptionVisitor.TYPE, new JavaxEjbApplicationExceptionVisitor<E>(
                ejbJarClassMetadata));

        // add @DeclareRoles
        getAnnotationVisitors().put(JavaxAnnotationSecurityDeclareRolesVisitor.TYPE,
                new JavaxAnnotationSecurityDeclareRolesVisitor<E>(ejbJarClassMetadata));

        // add @RolesAllowed
        getAnnotationVisitors().put(JavaxAnnotationSecurityRolesAllowedVisitor.TYPE,
                new JavaxAnnotationSecurityRolesAllowedVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @PermitAll
        getAnnotationVisitors().put(JavaxAnnotationSecurityPermitAllVisitor.TYPE,
                new JavaxAnnotationSecurityPermitAllVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

        // add @RunAs
        getAnnotationVisitors().put(JavaxAnnotationSecurityRunAs.TYPE,
                new JavaxAnnotationSecurityRunAs<E>(ejbJarClassMetadata));

        // add @ExcludeDefaultInterceptors
        getAnnotationVisitors().put(JavaxInterceptorExcludeDefaultInterceptorsVisitor.TYPE,
                new JavaxInterceptorExcludeDefaultInterceptorsVisitor<EjbJarClassMetadata<E>>(ejbJarClassMetadata));

    }

    /**
     * @param jField signature
     * @return configurator
     */
    @Override
    protected IFieldConfigurator createFieldMetadataConfigurator(final JField jField) {
        return new EjbJarFieldMetadataConfigurator<E>(jField, getClassMetadata(), this.annotationParsingDesactived);
    }

    /**
     * @param jMethod signature
     * @return configurator
     */
    @Override
    protected IMethodConfigurator createMethodMetadataConfigurator(final JMethod jMethod) {
        return new EjbJarMethodMetadataConfigurator<E>(jMethod, getClassMetadata(), this.annotationParsingDesactived);
    }

    /**
     * add class to archive.
     */
    @Override
    public void configurationComplete() {
       EjbJarClassMetadata<E> ejbJarClassMetadata = getClassMetadata();
       ejbJarClassMetadata.getEjbJarArchiveMetadata().addEjbJarClassMetadata(ejbJarClassMetadata);
    }

}
